<?php
namespace plugins\contactform\controllers;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

use plugins\antispam\library\InvisibleCaptcha;
use plugins\antispam\library\SimpleCaptcha;
use models\Template;
use \IoC;
use \Validator;

/**
 * Contact controller
 * @author Jérôme Brilland
 * @version 3.1
 * @copyright (c) 2021-2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class ContactController extends \ActionController {

    public function doIndex() {
        //antispam
        if ( !class_exists( 'plugins\\antispam\\library\\InvisibleCaptcha' ) ) {
            IoC::resolve( 'logger' )->error( 'Class \'plugins\\antispam\\library\\InvisibleCaptcha\' not found.' );
            $this->forward( '\\controllers\\ErrorController', '500' );
        }
        $invisibleCaptcha = new InvisibleCaptcha();
        $this->token = $invisibleCaptcha->generateToken( 'contact_form' );
        
        $simpleCaptcha = new SimpleCaptcha();
        $this->question = $simpleCaptcha->generateQuestion();

        $i18n = IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'contact_us' );

        $template = Template::findOne( ['name'], ['template_id' => IoC::resolve( 'settings' )['contactform.template']] );
        if ( isset( $template ) ) {            
            $latte = IoC::resolve( 'view' )->getLatte();
            $latte->getLoader()->addLoader( IoC::resolve( 'latteDatabaseLoader' ) );

            $this->render( $template->name );
        } else {
            $this->render( 'contactform.html' );
        }
    }
    
    public function doSend() {
        $i18n = IoC::resolve( 'i18n' );
        $errors = array();

        //honeypot
        if ( !empty( $this->request->getParameter( 'phone' ) ) ) {
            IoC::resolve( 'logger' )->warn( 'potential spam detected for IP ' . $_SERVER['REMOTE_ADDR'] );
            die;
        }
        $invisibleCaptcha = new InvisibleCaptcha();
        if ( !$invisibleCaptcha->checkToken( 'contact_form', $this->request->getParameters()->getAlnum( 'token' ) ) ) {
            IoC::resolve( 'logger' )->warn( 'potential spam detected for IP ' . $_SERVER['REMOTE_ADDR'] );
            die;
        }
        
        $simpleCaptcha = new SimpleCaptcha();
        if ( !$simpleCaptcha->checkAnswer( $this->request->getParameters()->getInt( 'antispam_answer' ) ) ) {
            $errors['antispam_answer'] = $i18n->__( 'wrong_antispam_answer' );
        }
        
        $name = htmlspecialchars( $this->request->getParameter( 'name' ), ENT_COMPAT | ENT_HTML5, 'UTF-8' );
        if ( !Validator::validateMinLength( $name, 3 ) ) {
            $errors['name'] = $i18n->__( 'contact.too_short_name', 3 );
        }
        if ( !Validator::validateMaxLength( $name, 100 ) ) {
            $errors['name'] = $i18n->__( 'contact.too_long_name', 100 );
        }
        if ( !Validator::validateNotEmpty( $name ) ) {
            $errors['name'] = $i18n->__( 'contact.name_is_required' );
        }
        $subject = htmlspecialchars( $this->request->getParameter( 'subject' ), ENT_COMPAT | ENT_HTML5, 'UTF-8' );
        if ( !Validator::validateMinLength( $subject, 5 ) ) {
            $errors['subject'] = $i18n->__( 'contact.too_short_subject', 5 );
        }
        if ( !Validator::validateMaxLength( $subject, 100 ) ) {
            $errors['subject'] = $i18n->__( 'contact.too_long_subject', 100 );
        }
        if ( !Validator::validateNotEmpty( $subject ) ) {
            $errors['subject'] = $i18n->__( 'contact.subject_is_required' );
        }
        $email = $this->request->getParameters()->getString( 'email', null, 'full_special_chars' );
        if ( !Validator::validateEmail( $email ) ) {
            $errors['email'] = $i18n->__( 'invalid_email' );
        }
        if ( !Validator::validateNotEmpty( $email ) ) {
            $errors['email'] = $i18n->__( 'email_is_required' );
        }
        $message = htmlspecialchars( $this->request->getParameter( 'message' ), ENT_COMPAT | ENT_HTML5, 'UTF-8' );
        if ( !Validator::validateNotEmpty( $message ) ) {
            $errors['message'] = $i18n->__( 'contact.message_is_required' );
        }
        
        if ( empty( $errors ) ) {
            $mail = new PHPMailer();
            $settings = IoC::resolve( 'settings' );
            $mail->AddAddress( $settings['contactform.to_email'] );
            $mail->Subject = $subject;
            $mail->Body = $message;
            $mail->From = $settings['from_email'];
            $mail->FromName = $name;
            $mail->addReplyTo( $email, $name );
            $mail->CharSet = PHPMailer::CHARSET_UTF8;
            $mail->IsMail();
            IoC::resolve( 'eventmediator' )->trigger( 'before_send_email', [$mail] );

            if( !$mail->Send() ) {
                $logger = IoC::resolve( 'logger' );
                $logger->error( $mail->ErrorInfo );
                $this->forward( '\\controllers\\ErrorController', '500' );
            } else {
                $this->flash['success'] = $i18n->__( 'email_successfully_sent' );
                $this->redirect( BASE_URL . 'contact' );
            }
        } else {
            $this->errors = $errors;
            $this->name = $name;
            $this->subject = $subject;
            $this->email = $email;
            $this->message = $message;
            $this->doIndex();
        }
    }

}

